@@ -43,6 +43,8 @@ get_variable(MPEquationParserState *state, const char *name, MPNumber *z)
 
     if (strcmp(name, "e") == 0)
         mp_get_eulers(z);
+    else if (strcmp(name, "i") == 0)
+        mp_get_i(z);
     else if (strcmp(name, "π") == 0)
         mp_get_pi(z);
     else if (state->options->get_variable)
@@ -118,9 +120,14 @@ sub_atoi(const char *data)
 int
 super_atoi(const char *data)
 {
-   int i, value = 0;
+   int i, sign = 1, value = 0;
    const char *digits[11] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹", NULL};
 
+   if(strncmp(data, "⁻", strlen("⁻")) == 0) {
+      sign = -1;
+      data += strlen("⁻");
+   }
+
    do {
       for(i = 0; digits[i] != NULL && strncmp(data, digits[i], strlen(digits[i])) != 0; i++);
       if(digits[i] == NULL)
@@ -129,7 +136,7 @@ super_atoi(const char *data)
       data += strlen(digits[i]);
    } while(*data != '\0');
 
-   return value;
+   return sign * value;
 }
 
 static int
@@ -235,10 +242,6 @@ mp_equation_parse(const char *expression, MPEquationOptions *options, MPNumber *
     _mp_equation__delete_buffer(buffer, yyscanner);
     _mp_equation_lex_destroy(yyscanner);
 
-    /* Failed to parse */
-    if (ret)
-        return PARSER_ERR_INVALID;
-
     /* Error during parsing */
     if (state.error)
         return state.error;
@@ -246,6 +249,10 @@ mp_equation_parse(const char *expression, MPEquationOptions *options, MPNumber *
     if (mp_get_error())
         return PARSER_ERR_MP;
 
+    /* Failed to parse */
+    if (ret)
+        return PARSER_ERR_INVALID;
+
     mp_set_from_mp(&state.ret, result);
 
     return PARSER_ERR_NONE;
@@ -267,6 +274,8 @@ mp_error_code_to_string(MPErrorCode error_code)
         return "PARSER_ERR_UNKNOWN_VARIABLE";
     case PARSER_ERR_UNKNOWN_FUNCTION:
         return "PARSER_ERR_UNKNOWN_FUNCTION";
+    case PARSER_ERR_UNKNOWN_CONVERSION:
+        return "PARSER_ERR_UNKNOWN_CONVERSION";
     case PARSER_ERR_MP:
         return "PARSER_ERR_MP";
     default:
